@@ -1,3 +1,20 @@
+# This file is part of Py6S.
+#
+# Copyright 2012 Robin Wilson and contributors listed in the CONTRIBUTORS file.
+#
+# Py6S is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Lesser General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# Py6S is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Lesser General Public License for more details.
+#
+# You should have received a copy of the GNU Lesser General Public License
+# along with Py6S.  If not, see <http://www.gnu.org/licenses/>.
+
 from collections import defaultdict
 #from sixs_exceptions import *
 
@@ -113,9 +130,10 @@ class AeroProfile:
       This returns an :class:`.AerosolDistribution` object. Components can then be added to this distribution using the :meth:`.add_component`
       method of the returned class.
       
-      Example usage:
-      s.aeroprofile = AeroProfile.JungePowerLawDistribution(0.3, 0.1)
-      s.aeroprofile.add_component(...)
+      Example usage::
+      
+        s.aeroprofile = AeroProfile.JungePowerLawDistribution(0.1, 0.3)
+        s.aeroprofile.add_component(...)
       
       """
       return cls.AerosolDistribution(rmin, rmax, 10)
@@ -286,4 +304,4 @@ class AeroProfile:
           res += "%f %f %d\n" % (val[0], val[1], self.aerotype)
         
         return res
-      
\ No newline at end of file
+      
